//original algorithm & code https://github.com/nathanchantrell/TinyPCRemote/blob/master/TinyPCRemote_CodeReader/TinyPCRemote_CodeReader.ino
//original head:

////----------------------------------------------------------------------------------------------------------------------
//// TinyPCRemote_CodeReader 
//// By Nathan Chantrell http://nathan.chantrell.net
//// Reads codes from the remote control and outputs to serial monitor
//// Enter the resulting codes in TinyPCRemote_ATtiny85.ino
////
//// Licenced under the Creative Commons Attribution-ShareAlike 3.0 Unported (CC BY-SA 3.0) licence:
//// http://creativecommons.org/licenses/by-sa/3.0/
////----------------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------------------
//modified and optimized for attiny13 by fishlabsoman 7.11.2014
//----------------------------------------------------------------------------------------------------------------------

#define IRpin_PIN PINB
#define IRpin 2

#define rLedPin 3
#define gLedPin 4
#define relayPin 1

#define MAXPULSE 5000
#define NUMPULSES 32
#define RESOLUTION 2

#define timeN1 1800000
#define timeN2 3600000

#define timerInterval 500


bool relayState = false;
unsigned long timer = 0;
unsigned long shift = timeN1;//30 min timer by default
unsigned long previousMillis = 0;
bool timerN = false;
byte i = 0;


void setup() {
  //default states
  DDRB |= (1<<relayPin);
  DDRB |= (1<<rLedPin);
  DDRB |= (1<<gLedPin);

  PORTB &= ~(1<<relayPin);//relay off
  PORTB &= ~(1<<rLedPin);//red led off
  PORTB |=  (1<<gLedPin);//green led on

  /*
  //for debug
  Serial.begin(9600);
  Serial.println("Start | "+String(millis()));
  //*/

  /*
  //for debug without ir receiver
  pinMode(5, INPUT);
  pinMode(6, INPUT);
  //*/


}

void shutDown(){
  relayState = true;
  PORTB |=  (1<<relayPin);
  PORTB &= ~(1<<gLedPin);
  PORTB |=  (1<<rLedPin);
  //Serial.println("turining off |"+String(millis()));
}

void startUp(){
  relayState = false;
  PORTB &= ~(1<<relayPin);
  PORTB |=  (1<<gLedPin);
  PORTB &= ~(1<<rLedPin);
  //Serial.println("turining on |"+String(millis()));
}

void loop() {
  unsigned long irCode = listenForIR(); // Wait for an IR Code
  //Serial.println("ir code: "+String(irCode));
  if(irCode == 3359105948){//green button
	//Serial.println("Pressed green btn |"+String(millis()));
	if(timer == 0){//on off mode
	  if(relayState == true){
		startUp();
	  }else{
		shutDown();
	  }
	}else{//cancel timer mode
	  timer = 0;
	  PORTB &= ~(1<<rLedPin);//turn off red led
	  //Serial.println("timer canceled |"+String(millis()));
	}
  }//end green btn


  if(3359101868 == irCode){//red btn
	//Serial.println("pressed red btn |"+String(millis()));
	if(timer == 0){
	  if(relayState == 0){
		timer = millis();
		//Serial.println("timer started |"+String(millis()));
	  }/*else{
		Serial.println("already shutdown |"+String(millis()));
	  }
	  //*/

	}else{//changing time mode
	  timerN = !timerN;
	  if(timerN){
		//Serial.println("change 20sec |"+String(millis()));
		shift = timeN1;//30 min
	  }else{
		//Serial.println("change 10sec |"+String(millis()));
		shift = timeN2;//30 min
	  }
	}
  }//end red btn
} // loop end

void checkTimer(){
  unsigned long time = millis();
  if(time - previousMillis >= timerInterval || previousMillis > time ) {
	previousMillis =time;
	timer1();
  }
}

unsigned long  listenForIR() {// IR receive code
  byte currentpulse = 0; // index for pulses we're storing
  unsigned long irCode = 0; // Wait for an IR Code
  irCode = irCode << 1;

  while (true) {
	unsigned int pulse = 0;// temporary storage timing
	//bool true (HIGH)
	while (IRpin_PIN & _BV(IRpin)) { // got a high pulse (99% standby time have HIGH)
	  if(++i > 150){//check timer every 150 iterations (high frequency break ir code timing)
		i = 0;
		checkTimer();
	  }
	  pulse++;
	  delayMicroseconds(RESOLUTION);
	  if (((pulse >= MAXPULSE) && (currentpulse != 0)) || currentpulse == NUMPULSES ) {
		return irCode;
	  }
	}

	//make irCode
	irCode = irCode << 1;
	if ((pulse * RESOLUTION) > 0 && (pulse * RESOLUTION) < 500) {
	  irCode |= 0;
	}else {
	  irCode |= 1;
	}
	currentpulse++;
	pulse = 0;
	//bool false (LOW)
	while (!(IRpin_PIN & _BV(IRpin))) {//wait before new pulse
	  //checkTimer();
	  pulse++;
	  delayMicroseconds(RESOLUTION);
	  if (pulse >= MAXPULSE || currentpulse == NUMPULSES ) {
		//Serial.println(irCode);
		return irCode;
	  }
	}
  }//end while(1)
  
}//end listenForIR 



//executing every timerInverval
void timer1() {
  if(timer != 0){
	if(timerN == true){//timeN1 or timeN2
	  PORTB |= (1<<rLedPin);
	}else{//blinking 30min	
	  PORTB ^= (1<<rLedPin);//invert
	}
	//Serial.println(String((timer+shift - millis())/1000));
  }

  if(timer != 0 &&(timer+shift < millis() || timer > millis())){
	timer = 0;
	shutDown();
  }
}

/*
//function for debug without ir receiver
unsigned long  listenForIR() {
 	//Serial.println("start without ir receiver |"+String(millis()));
 	delay(350);
 	while(true){
 
 		checkTimer();
 
 		if(PIND & _BV(5)){
 			while(PIND & _BV(5)){

				//Serial.print("g");
 				//wait
 			}
 			return 3359101868;
 		}
 
 		if(PIND & _BV(6)){
 			while(PIND & _BV(6)){
 
				//Serial.print("r");
 				//wait
 			}
 			return 3359105948;
 		}
 	}
 	return 0;
 }
 //*/




